module gutil

!
!  Modified
!	German Prieto
!	March 2005
!
!  This file contains a module with specific functions
!  and subroutines to be called by any program. It's 
!  kind of a subroutine repository. The use of this
!  module is to be able to set up subroutines and 
!  functions that deal with integer, real, complex 
!  numbers, by calling the same subroutine or function.
!  Based on the idea of Numerical Recipes for F90.
!
!
! We use here interfaces, to allow overloading. Overloading is
! used for example to use the same function for different types  
! of variables. For example to sum two integers, or sum two reals,
! one would need two different functions. By overloading, you just 
! have to call one.
!

! 
! Generic interfaces for routines with overload versions. Naming 
! conventions for appended codes in the names of overloaded routines
! are as follows:
!
! r = real
! d = double precision
! i = integer
! c = complex
! z = double precision complex
! h = character
! l = logical
! 
! Any of these might be followed by
!
! v = vector
! m = matrix
! only used to resolve ambiguities.
!

!
!  Table of contents
!
!
!
!  routine to meshgrid
!	meshgrid
!
!

!
!  Interfaces
!

   interface meshgrid
      module procedure meshgrid_i, meshgrid_r, meshgrid_d,      &
                   meshgrid_c, meshgrid_z
   end interface meshgrid

!
!  The subroutines
!

   contains
   
!--------------------------------------------------------------------
!  Meshgrid
!--------------------------------------------------------------------

   subroutine meshgrid_i(x,y,XX,YY)

   !
   ! create a meshgrid (just as matlab) of vectors x and y
   !

   !*****************************************************************
   
      implicit none

      integer :: xn, yn 
      
      integer, dimension(:), intent(in) :: x, y

      integer, dimension(:,:), intent(out) :: XX,YY


   !*****************************************************************

      xn = size(x,1)
      yn = size(y,1)

      XX = spread(x, dim=1, ncopies=yn )

      YY = spread(y, dim=2, ncopies=xn )

      return

   end subroutine meshgrid_i

!--------------------------------------------------------------------

   subroutine meshgrid_r(x,y,XX,YY)

   !
   ! create a meshgrid (just as matlab) of vectors x and y
   !

   !*****************************************************************
   
      implicit none

      integer :: xn, yn 
      
      real, dimension(:), intent(in) :: x, y

      real, dimension(:,:), intent(out) :: XX,YY


   !*****************************************************************

      xn = size(x,1)
      yn = size(y,1)

      XX = spread(x, dim=1, ncopies=yn )

      YY = spread(y, dim=2, ncopies=xn )

      return

   end subroutine meshgrid_r

!--------------------------------------------------------------------

   subroutine meshgrid_d(x,y,XX,YY)

   !
   ! create a meshgrid (just as matlab) of vectors x and y
   !

   !*****************************************************************
   
      implicit none

      integer :: xn, yn 
      
      real(8), dimension(:), intent(in) :: x, y

      real(8), dimension(:,:), intent(out) :: XX,YY


   !*****************************************************************

      xn = size(x,1)
      yn = size(y,1)

      XX = spread(x, dim=1, ncopies=yn )

      YY = spread(y, dim=2, ncopies=xn )

      return

   end subroutine meshgrid_d

!--------------------------------------------------------------------

   subroutine meshgrid_c(x,y,XX,YY)

   !
   ! create a meshgrid (just as matlab) of vectors x and y
   !

   !*****************************************************************
   
      implicit none

      integer :: xn, yn 
      
      complex, dimension(:), intent(in) :: x, y

      complex, dimension(:,:), intent(out) :: XX,YY


   !*****************************************************************

      xn = size(x,1)
      yn = size(y,1)

      XX = spread(x, dim=1, ncopies=yn )

      YY = spread(y, dim=2, ncopies=xn )

      return

   end subroutine meshgrid_c

!--------------------------------------------------------------------

   subroutine meshgrid_z(x,y,XX,YY)

   !
   ! create a meshgrid (just as matlab) of vectors x and y
   !

   !*****************************************************************
   
      implicit none

      integer :: xn, yn 
      
      complex(8), dimension(:), intent(in) :: x, y

      complex(8), dimension(:,:), intent(out) :: XX,YY


   !*****************************************************************

      xn = size(x,1)
      yn = size(y,1)

      XX = spread(x, dim=1, ncopies=yn )

      YY = spread(y, dim=2, ncopies=xn )

      return

   end subroutine meshgrid_z


end module gutil

