module spectra

!
!  Interface to allow for x or y to be a matrix, and plot each vector
!  of y against x. The matrix is given by horizontal vectors, y(:,1) is 
!  the first vector.
!

!
!  Interface
!      		d	double precision version
!		r	single precision version
!               m	matrix, single precision
!

   interface mtspec
      module procedure mtspec_d, mtspec_r
   end interface mtspec

!
!  The subroutines
!		
!	The original subroutine performs computation in double precision, 
!	sometimes this is not needed. So I created this interface to 
!	allow single precision output. Be aware that all computations 
!	are still double precision, only the input/output is going
!	to be single precision.
!
!
!  Modified
!	German Prieto
!	April 2005
!
!  Based on David J. Thomson's Codes, Chave and Thomson's Codes and
!  partial codes from EISPACK, Robert L. Parker and Glenn Ierley from
!  Scripps Institution of Oceanography.
!
!  The subroutine is in charge of estimating the adaptive weigthed 
!  multitaper spectrum, as in Thomson 1982. 
!  This is done by estimating the dpss (discrete prolate spheroidal 
!  sequences, multiplying each of the kspec taper with the data 
!  series, take the fft, and using the adaptive scheme for a better
!  estimation. 
!  As a by product of the spectrum (spec), all intermediate steps 
!  are estimated, and can be called as optional variables (see 
!  documentation). By products include the phase information
!  in yk, the eigenspectra sk, the simple weigthed mean spectra sbar, 
!  the jackknife 95% confidence intervals (err), the degrees of freedom 
!  (se) and the weigths wt(nf,kspec) used.
!  To get ahold of this values, simply call them in the subroutine. Note
!  that the order of calling this variables does matter. But if you want 
!  specific variables from the subroutine you can specify them (see 
!  documentation). 
!
!  Variable names used in the subroutine
!
!   	xmean	mean of the time series (data is demeaned)
!	xvar	variance of time series
!   	df	frequency bin separation 
!	fnyq	Nyquist frequency
!	sk	eigenspectra (yk**2)
!   	yk 	eigencoefficients
!	sbar	mean spectrum
!	se 	number of degrees of freedom for each frequency bin
!	err	jackknife 95% confidence interval
!	wt	array containing the ne weights for kspec eigenspectra 
!		normalized so that the sum of squares over
!           	the kspec eigenspectra is one
!	lambda  eigenvalues of the dpss
!	theta	1 - eigenvalues
!	vn	dpss sequences
!	xi 	Variance efficiency Eq. 7.2(c)
!	seavg	Stability of estimate (should be close to unity) Eq. 5.5
!
!
!  Input
!
!	npts	-	integer number of data points in x
!	dt	- 	real, sampling rate of the time series x
!	x(npts) -	real, data vector
!	tbp	-	real, the time-bandwidth product (NW)
!	kspec	- 	integer, number of tapers to use in the estimation
!	nf	-	integer, number of frequency bins (npts/2 + 1)
!
!  Output
!
!	freq(nf)	real vector with frequency bins
!	spec(nf)	real vector with the adaptive estimated spectrum
!
!  Optional Output
!
!	sk	eigenspectra (yk**2)
!   	yk 	eigencoefficients
!	sbar	mean spectrum
!	se 	number of degrees of freedom for each frequency bin
!	err	jackknife 95% confidence interval
!	wt	array containing the ne weights for kspec eigenspectra 
!		normalized so that the sum of squares over
!           	the kspec eigenspectra is one
!	lambda  eigenvalues of the dpss
!	theta	1 - eigenvalues
!	vn	dpss sequences
!	xi 	Variance efficiency Eq. 7.2(c)
!	seavg	Stability of estimate (should be close to unity) Eq. 5.5

!
!  calls
!	dpss, eigenft, adaptspec
!

contains

subroutine mtspec_d (npts,dt,x,tbp,kspec,nf,freq,spec,        	&
		yk, wt, err, se, sk, sbar, vn, lambda, theta,   &
		xi, seavg )

!
!  The double precision version. This is the original version of the 
!  subroutine.
!

!**********************************************************************

   implicit none

!  Input

   integer, intent(in) :: npts, nf, kspec

   real(8), intent(in) :: dt, tbp

   real(8), intent(in), dimension(npts) :: x

!  Output

   real(8), intent(out), dimension(nf) :: freq

   real(8), intent(out), dimension(nf) :: spec

!
!  Optional
!

!  Eigenspectra

   real(8), dimension(nf,kspec) 	  :: sk_o 
   real(8), dimension(nf,kspec), optional :: sk
   
   complex(8), dimension(npts,kspec) 	       :: yk_o 
   complex(8), dimension(npts,kspec), optional :: yk 

!  Simple mean spectra

   real(8), dimension(nf) 	    :: sbar_o
   real(8), dimension(nf), optional :: sbar

!  Adaptive spectra

   real(8), dimension(nf) 	    :: se_o 
   real(8), dimension(nf), optional :: se

   real(8), dimension(nf,kspec)	 	  :: wt_o
   real(8), dimension(nf,kspec), optional :: wt

   real(8), dimension(nf,2) 	      :: err_o
   real(8), dimension(nf,2), optional :: err
   
   
!  Dpss 

   real(8), dimension(kspec)	       :: lambda_o, theta_o
   real(8), dimension(kspec), optional :: lambda, theta
   
   real(8), dimension(npts,kspec)	    :: vn_o
   real(8), dimension(npts,kspec), optional :: vn

!  Efficiency and Stability of estimate

   real(8)	     :: xi_o, seavg_o
   real(8), optional :: xi, seavg

!
!  Extras
!

!  Time series

   real(8) :: xmean, xvar 

!  Frequency variables

   real(8) :: df, fnyq

!  Other

   integer :: i, j

   real(8) :: xsum, sscal

   real(8), dimension(npts) :: x2

!********************************************************************

   write(6,'(a)') 'Calling double precision'

!
! Frequency variables
!

   fnyq	= 0.5d0/dt
   df 	= fnyq/dble(nf - 1)

!
!  Define frequency bins
!

   do i = 1,nf
      freq(i) = dble(i-1)*df
   enddo
 
!
!  Get time series stats (mean and variance)
!  Always demean the time series
!

   xmean = sum(x)/dble(npts)
   xvar = (sum((x - xmean)**2))/dble(npts)

   x2 = x - xmean

!
!  Get the dpss
!

   call dpss(npts,tbp,kspec,vn_o,lambda_o,theta_o)

   write(6,'(3x,a/(4f18.14))')'Prolate spheroidal eigenvalues:',  &
     			       (lambda_o(j),j=1,kspec)

!
! Get the eigenspectra (yk's, sk's and smean)
!

   call eigenft(npts, x2, kspec, vn_o, lambda_o, nf, yk_o, sk_o, sbar_o)

!
!  Calcuate adaptive weigthed spectrum
!

   call adaptspec(npts,nf,kspec,xvar,theta_o,yk_o, &
                spec,se_o,wt_o,err_o)

!
!  double power in positive frequencies
!

   do i=2,nf
   
      spec(i) = 2.d0 * spec(i)
   
   enddo

!
!  scale spectrum to meet parseval's theorem
!
   
   sscal = dble(npts)*xvar/sum(spec)
      
   spec = sscal*spec

!
!  Put errors on scale
!

   err_o(:,1) = spec / err_o(:,1)
   err_o(:,2) = spec * err_o(:,2)
  
!
!  Average stability (average degrees of freedom)
!		(Eq. 5.5 Pg 1065)
!

   seavg_o=sum(se_o)/(2.d0*dble(kspec)*dble(nf))
   
   write(6,'(a,g15.7)') ' Average stability of estimate = ', seavg_o

!
!  compute estimate of variance efficiency 
!		(Eq 7.2(c) Pg 1069)
!
   
   xi_o=0.
   do i=1,npts
      xsum=0.
      do j=1,kspec
         xsum = xsum+vn_o(i,j)**2
      enddo
      xsum = xsum/dble(kspec)
      xi_o = xi_o+xsum**2
   enddo
      
   xi_o=1./(dble(npts)*xi_o)

   write(6,'(a,g15.7)') ' Variance efficiency = ', xi_o

!
!  Output the appropiate optional variables
!

   if (present(yk)) then
      yk = yk_o
   endif
   if (present(wt)) then
      wt = wt_o
   endif
   if (present(err)) then
      err = err_o
   endif
   if (present(se)) then
      se = se_o
   endif
   if (present(sk)) then
      sk = sk_o
   endif
   if (present(sbar)) then
      sbar = sbar_o
   endif
   if (present(vn)) then
      vn = vn_o
   endif
   if (present(lambda)) then
      lambda = lambda_o
   endif
   if (present(theta)) then
      theta = theta_o
   endif
   if (present(xi)) then
      xi = xi_o
   endif
   if (present(seavg)) then
      seavg = seavg_o
   endif

end subroutine mtspec_d

!--------------------------------------------------------------------
! The real*4 version
!--------------------------------------------------------------------


subroutine mtspec_r (npts,dt,x,tbp,kspec,nf,freq,spec,        	&
		yk, wt, err, se, sk, sbar, vn, lambda, theta,   &
		xi, seavg )

!
!  This is the single precision version of the subroutine. It is 
!  simply the original version, with the output changed to real(4).
!

!**********************************************************************

   implicit none

!  Input

   integer, intent(in) :: npts, nf, kspec

   real(4), intent(in) :: dt, tbp

   real(4), intent(in), dimension(npts) :: x

!  Output

   real(4), intent(out), dimension(nf) :: freq

   real(4), intent(out), dimension(nf) :: spec
   real(8), dimension(nf) :: spec8

!
!  Optional
!

!  Eigenspectra

   real(8), dimension(nf,kspec) 	  :: sk_o 
   real(4), dimension(nf,kspec), optional :: sk
   
   complex(8), dimension(npts,kspec) 	       :: yk_o 
   complex(4), dimension(npts,kspec), optional :: yk 

!  Simple mean spectra

   real(8), dimension(nf) 	    :: sbar_o
   real(4), dimension(nf), optional :: sbar

!  Adaptive spectra

   real(8), dimension(nf) 	    :: se_o 
   real(4), dimension(nf), optional :: se

   real(8), dimension(nf,kspec)	 	  :: wt_o
   real(4), dimension(nf,kspec), optional :: wt

   real(8), dimension(nf,2) 	      :: err_o
   real(4), dimension(nf,2), optional :: err
   
   
!  Dpss 

   real(8), dimension(kspec)	       :: lambda_o, theta_o
   real(8), dimension(kspec), optional :: lambda, theta
   
   real(8), dimension(npts,kspec)	    :: vn_o
   real(8), dimension(npts,kspec), optional :: vn

!  Efficiency and Stability of estimate

   real(8)	     :: xi_o, seavg_o
   real(4), optional :: xi, seavg

!
!  Extras
!

!  Time series

   real(8) :: xmean, xvar 

!  Frequency variables

   real(8) :: df, fnyq

!  Other

   integer :: i, j

   real(8) :: xsum, sscal

   real(8), dimension(npts) :: x2

!********************************************************************

   write(6,'(a)') 'Calling single precision'

!
! Frequency variables
!

   fnyq	= 0.5d0/dble(dt)
   df 	= fnyq/dble(nf - 1)

!
!  Define frequency bins
!

   do i = 1,nf
      freq(i) = real(i-1)*real(df)
   enddo
  
!
!  Get time series stats (mean and variance)
!  Always demean the time series
!

   xmean = sum(dble(x))/dble(npts)
   xvar = (sum((dble(x) - xmean)**2))/dble(npts)

   x2 = dble(x) - xmean

!
!  Get the dpss
!

   call dpss(npts,dble(tbp),kspec,vn_o,lambda_o,theta_o)

   write(6,'(3x,a/(4f18.14))')'Prolate spheroidal eigenvalues:',  &
     			       (lambda_o(j),j=1,kspec)

!
! Get the eigenspectra (yk's, sk's and smean)
!

   call eigenft(npts, x2, kspec, vn_o, lambda_o, nf, yk_o, sk_o, sbar_o)

!
!  Calcuate adaptive weigthed spectrum
!

   call adaptspec(npts,nf,kspec,xvar,theta_o,yk_o, &
                spec8,se_o,wt_o,err_o)

!
!  double power in positive frequencies
!

   do i=2,nf
   
      spec8(i) = 2.d0 * spec8(i)
   
   enddo

!
!  scale spectrum to meet parseval's theorem
!
   
   sscal = dble(npts)*xvar/sum(spec8)
      
   spec = real(sscal*spec8)

!
!  Put errors on scale
!

   err_o(:,1) = spec8 / err_o(:,1)
   err_o(:,2) = spec8 * err_o(:,2)
  
!
!  Average stability (average degrees of freedom)
!		(Eq. 5.5 Pg 1065)
!

   seavg_o=sum(se_o)/(2.d0*dble(kspec)*dble(nf))
   
   write(6,'(a,g15.7)') ' Average stability of estimate = ', seavg_o

!
!  compute estimate of variance efficiency 
!		(Eq 7.2(c) Pg 1069)
!
   
   xi_o=0.
   do i=1,npts
      xsum=0.
      do j=1,kspec
         xsum = xsum+vn_o(i,j)**2
      enddo
      xsum = xsum/dble(kspec)
      xi_o = xi_o+xsum**2
   enddo
      
   xi_o=1./(dble(npts)*xi_o)

   write(6,'(a,g15.7)') ' Variance efficiency = ', xi_o

!
!  Output the appropiate optional variables
!

   if (present(yk)) then
      yk = cmplx(yk_o)
   endif
   if (present(wt)) then
      wt = real(wt_o)
   endif
   if (present(err)) then
      err = real(err_o)
   endif
   if (present(se)) then
      se = real(se_o)
   endif
   if (present(sk)) then
      sk = real(sk_o)
   endif
   if (present(sbar)) then
      sbar = real(sbar_o)
   endif
   if (present(vn)) then
      vn = (vn_o)
   endif
   if (present(lambda)) then
      lambda = (lambda_o)
   endif
   if (present(theta)) then
      theta = (theta_o)
   endif
   if (present(xi)) then
      xi = real(xi_o)
   endif
   if (present(seavg)) then
      seavg = real(seavg_o)
   endif

end subroutine mtspec_r


end module spectra

